@@ -29,8 +29,7 @@ static struct otx2_cpt_bitmap get_cores_bmap(struct device *dev,
 	bool found = false;
 	int i;
 
-	if (eng_grp->g->engs_num < 0 ||
-	    eng_grp->g->engs_num > OTX2_CPT_MAX_ENGINES) {
+	if (eng_grp->g->engs_num > OTX2_CPT_MAX_ENGINES) {
 		dev_err(dev, "unsupported number of engines %d on octeontx2\n",
 			eng_grp->g->engs_num);
 		return bmap;
@@ -1419,7 +1418,6 @@ int otx2_cpt_discover_eng_capabilities(struct otx2_cptpf_dev *cptpf)
 	dma_addr_t rptr_baddr;
 	struct pci_dev *pdev;
 	u32 len, compl_rlen;
-	int timeout = 10000;
 	int ret, etype;
 	void *rptr;
 
@@ -1484,27 +1482,16 @@ int otx2_cpt_discover_eng_capabilities(struct otx2_cptpf_dev *cptpf)
 							 etype);
 		otx2_cpt_fill_inst(&inst, &iq_cmd, rptr_baddr);
 		lfs->ops->send_cmd(&inst, 1, &cptpf->lfs.lf[0]);
-		timeout = 10000;
 
 		while (lfs->ops->cpt_get_compcode(result) ==
-						OTX2_CPT_COMPLETION_CODE_INIT) {
+						OTX2_CPT_COMPLETION_CODE_INIT)
 			cpu_relax();
-			udelay(1);
-			timeout--;
-			if (!timeout) {
-				ret = -ENODEV;
-				cptpf->is_eng_caps_discovered = false;
-				dev_warn(&pdev->dev, "Timeout on CPT load_fvc completion poll\n");
-				goto error_no_response;
-			}
-		}
 
 		cptpf->eng_caps[etype].u = be64_to_cpup(rptr);
 	}
+	dma_unmap_single(&pdev->dev, rptr_baddr, len, DMA_BIDIRECTIONAL);
 	cptpf->is_eng_caps_discovered = true;
 
-error_no_response:
-	dma_unmap_single(&pdev->dev, rptr_baddr, len, DMA_BIDIRECTIONAL);
 free_result:
 	kfree(result);
 lf_cleanup:
